Con

  _CLKMODE = XTAL1 + PLL16X
  _CLKFREQ = 80_000_000

  MOTOR_L_RST   = 24
  MOTOR_L_FAULT = 3

  MOTOR_L_BOT   = 4
  MOTOR_L_TOP   = 7

  REVERSE_L_ON  = 4             ' ALO
  REVERSE_L_PWM = 5             ' BHI
  FORWARD_L_ON  = 6             ' BLO
  FORWARD_L_PWM = 7             ' AHI

  MOTOR_R_RST   = 11
  MOTOR_R_FAULT = 12

  MOTOR_R_BOT   = 13
  MOTOR_R_TOP   = 16

  FORWARD_R_PWM = 13            ' AHI
  REVERSE_R_ON  = 14            ' ALO
  REVERSE_R_PWM = 15            ' BHI
  FORWARD_R_ON  = 16            ' BLO

  FREQUENCY     = 20_000
  MAX_ON_TIME   = _CLKFREQ / FREQUENCY

Var

   byte ID


Dat

Structure
LeftPower     long      0
RightPower    long      0


Pub Start

  result := ID := cognew(@Entry, @Structure)
     

Pub Stop

  cogstop(ID)


Pub Left(Value)

  return LeftPower := Value


Pub Right(Value)

  return RightPower := Value                        


Pub Halt

  LeftPower~
  RightPower~


Dat
              org
Entry

              mov       dira, OutputsMask      ' set outputs
              mov       outa, ResetsMask
              
              mov       frqa, #1                ' frq is added to phs every clock cycle
              mov       frqb, #1                ' so it takes -phs clockcycles to generate a falling edge

              mov       NextCnt, cnt            ' initialize NextCnt
              add       NextCnt, HalfPeriod
Loop
              mov       Address, par            ' Read this side's motor power level
              rdlong    MotorLPower, Address                                                  

              abs       MotorLPower, MotorLPower wc ' nc = forward, c = reverse
              max       MotorLPower, MaxOnTime  ' Limit the maximum pulse width

              waitcnt   NextCnt, HalfPeriod     ' Wait for the end of the last pulse on this side
              mov       NextCtr, CTR_Mode       ' Reset counter A to the correct mode with a null pin value
        if_nc or        NextCtr, Fwd_L_PWM      ' Set the counter A output pin appropriatly 
        if_c  or        NextCtr, Rev_L_PWM      '  based on the travel of direction

        if_nc muxc      outa, Rev_L_On_Pin      ' If moving forward, clear this side's reverse-on pin
        if_c  muxnc     outa, Fwd_L_On_Pin      ' If moving in reverse, clear this side's forward-on pin
        if_nc muxnc     outa, Fwd_L_On_Pin      ' If moving forward, set this side's forward-on pin
        if_c  muxc      outa, Rev_L_On_Pin      ' If moving in reverse, set this side's reverse-on pin

              mov       ctra, NextCtr
              neg       phsa, MotorLPower ' Start the pulse

              add       Address, #4             ' Read this side's motor power level
              rdlong    MotorRPower, Address                                                  

              abs       MotorRPower, MotorRPower wc ' nc = forward, c = reverse
              max       MotorRPower, MaxOnTime  ' Limit the maximum pulse width

              waitcnt   NextCnt, HalfPeriod     ' Wait for the end of the last pulse on this side
              mov       NextCtr, CTR_Mode       ' Reset counter A to the correct mode with a null pin value
        if_nc or        NextCtr, Fwd_R_PWM      ' Set the counter A output pin appropriatly 
        if_c  or        NextCtr, Rev_R_PWM      ' based on the travel of direction

        if_nc muxc      outa, Rev_R_On_Pin      ' If moving forward, clear this side's reverse-on pin
        if_c  muxnc     outa, Fwd_R_On_Pin      ' If moving in reverse, clear this side's forward-on pin
        if_nc muxnc     outa, Fwd_R_On_Pin      ' If moving forward, set this side's forward-on pin
        if_c  muxc      outa, Rev_R_On_Pin      ' If moving in reverse, set this side's reverse-on pin

              mov       ctrb, NextCtr
              neg       phsb, MotorRPower ' Start the pulse

              jmp       #Loop

HalfPeriod    long      _CLKFREQ / (FREQUENCY * 2)
MaxOnTime     long      _CLKFREQ / FREQUENCY
L_Reset_Pin   long      |< MOTOR_L_RST
Fwd_L_On_Pin  long      |< FORWARD_L_ON
Fwd_L_PWM     long      FORWARD_L_PWM
Rev_L_On_Pin  long      |< REVERSE_L_ON
Rev_L_PWM     long      REVERSE_L_PWM
R_Reset_Pin   long      |< MOTOR_R_RST
Fwd_R_On_Pin  long      |< FORWARD_R_ON
Fwd_R_PWM     long      FORWARD_R_PWM
Rev_R_On_Pin  long      |< REVERSE_R_ON
Rev_R_PWM     long      REVERSE_R_PWM

OutputsMask   long      |< MOTOR_L_RST + |< FORWARD_L_ON + |< FORWARD_L_PWM + |< REVERSE_L_ON + |< REVERSE_L_PWM {
                        } + |< MOTOR_R_RST + |< FORWARD_R_ON + |< FORWARD_R_PWM + |< REVERSE_R_ON + |< REVERSE_R_PWM
ResetsMask    long      |< MOTOR_L_RST + |< MOTOR_R_RST
CTR_Mode      long      %00100 << 26


MotorLPower   res       1
MotorRPower   res       1
NextCnt       res       1
NextCtr       res       1
Address       res       1


             
                                                                              